﻿/*
 * MultiSegmentDisplay.cpp
 *
 * Created: 2016-01-20 오후 10:38:19
 *  Author: lsw2000
 */ 

#include "MultiSegmentDisplay.h"
#include <avr/interrupt.h>

/*******************************************************************************
 LED display data (Like the frame buffer)
*******************************************************************************/
uint16_t MultiSegmentDisplay::tick_timer = 0;
uint8_t MultiSegmentDisplay::segmentOutBuffer[MAX_SEGMENT_COUNT];
uint8_t MultiSegmentDisplay::segmentAttributeBuffer[MAX_SEGMENT_COUNT];

uint8_t MultiSegmentDisplay::barGraphSegmentOutBuffer[MAX_BAR_SEGMENT_COUNT];
uint8_t MultiSegmentDisplay::barGraphSegmentAttributeBuffer[MAX_BAR_SEGMENT_COUNT];

uint8_t MultiSegmentDisplay::maxAcceleratorIndex = 0;
uint16_t MultiSegmentDisplay::maxAcceleratorIndexUpdateTickTime = 0;
bool MultiSegmentDisplay::startMoveDown = false;

uint8_t MultiSegmentDisplay::downShiftPointSegmentIndex = 0;
uint8_t MultiSegmentDisplay::downShiftPointOffsetIndex = 0;
bool MultiSegmentDisplay::downShiftPointDimming = true;

/*******************************************************************************
*******************************************************************************/
MultiSegmentDisplay::MultiSegmentDisplay()
{
	clearAll();
	initTimer0ForSegmentRefresh();
	initTimer2ForAttributeUpdate();
	initPort();
}

/*******************************************************************************
*******************************************************************************/
void MultiSegmentDisplay::clearAll()
{
	for (uint8_t i = 0; i < MAX_SEGMENT_COUNT; i++) {
		segmentOutBuffer[i] = ~0;
		segmentAttributeBuffer[i] = 0;
	}
	for (uint8_t i = 0; i < MAX_BAR_SEGMENT_COUNT; i++) {
		barGraphSegmentOutBuffer[i] = ~0;
		barGraphSegmentAttributeBuffer[i] = 0;
	}
}

/*******************************************************************************
*******************************************************************************/
void MultiSegmentDisplay::setSegmentData(uint8_t bufferIndex, uint8_t rawValue)
{
	segmentOutBuffer[bufferIndex < MAX_SEGMENT_COUNT ? bufferIndex : MAX_SEGMENT_COUNT -1] = rawValue;
}

/*******************************************************************************
*******************************************************************************/
uint8_t MultiSegmentDisplay::getRawValueFromOneDigitValue(uint8_t oneDigitValue)
{
	return charMappingRawData[oneDigitValue < 10 ? oneDigitValue : 9];
}

/*******************************************************************************
 '0' ~ '9'		: 48 ~ 57
 : ; < = > ? @	: 58 ~ 64
 'A' ~ 'Z'		: 65 ~ 90
 [ \ ] ^ _ '	: 91 ~ 96
 'a' ~ 'z'		: 97 ~ 122
 { | }			: 123 ~ 125
*******************************************************************************/
uint8_t MultiSegmentDisplay::getRawValueFromOneChar(char aChar)
{
	uint8_t result = RAW_BLANK;
	if (aChar >= '0' && aChar <= '}') {
		result = charMappingRawData[aChar - '0'];
	} else if (aChar == '+') {
		result = RAW_PLUS;
	} else if (aChar == '-') {
		result = RAW_MINUS;
	}
	return result;
}

/*******************************************************************************
 Just for positive 3 digit value display with dot position
 Special for TPMS display ( 32.1 )
*******************************************************************************/
void MultiSegmentDisplay::set3DigitValueWithDotPos(uint8_t bufferIndex, uint16_t digit3Value, uint8_t dotPosition)
{
	//for (int i = 0; i < 3; i++) {
		//uint8_t aDigit = digit3Value % 10;
		//digit3Value /= 10;
		//uint8_t pos = bufferIndex + 2 - i;
		//if (pos >= 15) {
			//pos = 14;
		//}
		//
		//uint8_t rawData = charMappingRawData[aDigit];
		//if (dotPosition == i) {
			//rawData &= RAW_DOT;
		//}
		//setSegmentData(pos, rawData);
	//}
	
	uint8_t pos = 0;
	uint8_t digit = digit3Value / 100 % 10;
	bool digitZero = false;
	digitZero = (digit == 0);
	uint8_t rawData = digitZero ? RAW_BLANK : charMappingRawData[digit];
	if (pos == dotPosition)
		rawData &= RAW_DOT;
	setSegmentData(bufferIndex + pos++, rawData);
	
	digit = digit3Value / 10 % 10;
	if (digitZero) {
		digitZero = (digit == 0);
	}
	rawData = digitZero ? RAW_BLANK : charMappingRawData[digit];
	if (pos == dotPosition)
	rawData &= RAW_DOT;
	setSegmentData(bufferIndex + pos++, rawData);
	
	digit = digit3Value % 10;
	rawData = charMappingRawData[digit];
	if (pos == dotPosition)
	rawData &= RAW_DOT;
	setSegmentData(bufferIndex + pos++, rawData);
}

/*******************************************************************************

*******************************************************************************/
void MultiSegmentDisplay::setString(uint8_t bufferIndex, const char* string)
{
	for (int i = bufferIndex; i < MAX_SEGMENT_COUNT; i++) {
		char aChar = *(string++);
		if (!aChar) {
			return;
		}
		segmentOutBuffer[bufferIndex++] = getRawValueFromOneChar(aChar);
	}
}

/*******************************************************************************
 String to raw data mapping data
*******************************************************************************/
uint8_t MultiSegmentDisplay::charMappingRawData['}' - '0'+ 1] = {
	RAW_0, RAW_1, RAW_2, RAW_3, RAW_4, RAW_5, RAW_6, RAW_7, RAW_8, RAW_9,						// '0' ~ '9'
	RAW_COLON, RAW_SEMICOLON, RAW_SMALL, RAW_EQUAL, RAW_GREAT, RAW_QUESTION, RAW_AT,			// ':' ~ '@'
	RAW_A, RAW_B, RAW_C, RAW_D, RAW_E, RAW_F, RAW_G, RAW_H, RAW_I, RAW_J, RAW_K, RAW_L, RAW_M,	// 'A' ~ 'Z'
	RAW_N, RAW_O, RAW_P, RAW_Q, RAW_R, RAW_S, RAW_T, RAW_U, RAW_V, RAW_W, RAW_X, RAW_Y, RAW_Z,
	RAW_SQUARE_BRACKET_LEFT, RAW_BACK_SLASH, RAW_SQUARE_BRACKET_RIGHT, RAW_HAT, RAW_UNDER_LINE, RAW_SINGLE_QUOTATION, // '[' ~ "'"
	RAW_A, RAW_B, RAW_C, RAW_D, RAW_E, RAW_F, RAW_G, RAW_H, RAW_I, RAW_J, RAW_K, RAW_L, RAW_M,	// 'a' ~ 'z'
	RAW_N, RAW_O, RAW_P, RAW_Q, RAW_R, RAW_S, RAW_T, RAW_U, RAW_V, RAW_W, RAW_X, RAW_Y, RAW_Z,
	RAW_SQUARE_BRACKET_LEFT, RAW_BAR, RAW_SQUARE_BRACKET_RIGHT									// '{', '|', '}'
};

/*******************************************************************************
 7 Segment update timer initialization
*******************************************************************************/
void MultiSegmentDisplay::initTimer0ForSegmentRefresh()
{
	cli();
	TCCR0A = _BV(WGM01) | _BV(CS01) | _BV(CS00);	// 0b00001011WGM01 for CTC and Clock bit 011 (/64) 
	OCR0A = 64;	//125;		// 16,000,000 / 64 / x = 2,000 | x = 125
	TIMSK0 = _BV(OCIE0A);	// 0b10 – OCIE0A: Timer/Counter0 Output Compare Match A Interrupt Enable
	sei();
}

/*******************************************************************************
 Setup the timer 2 for update the attribute with timeout
*******************************************************************************/
void MultiSegmentDisplay::initTimer2ForAttributeUpdate()
{
	// Long term timer for attribute time out looping logic
	// 16MHz 0.0625us to /1024 then  --> 15,625 Hz
	// / 255(OCR2A) then 61.03 Hz
	// / 156(OCR2A) then 100.16 Hz
	cli();
	TCCR2A =0b00000111;				// /1024  [CS22,CS21,CS20] = 111
	OCR2A = 155;					// /156 then 100.16Hz --> + 10 count then 10Hz
	TIMSK2 = TIMSK2 | (1<< OCIE2A);	// Invoke interrupt on OCR2A matched
	sei();
}

/*******************************************************************************
 Initialize PORT
*******************************************************************************/
void MultiSegmentDisplay::initPort()
{
	DDRA = 0xFF;	// Port A output for digit selection first 8 digit - Common Anode(+) output
	DDRC = 0xFF;	// Port C output for digit selection next 7 digit
	DDRF = 0xFF;	// Port F output for 7 segment(+Comma) displaying
	
	// Test code for bar graph display
	DDRB = 0xFF;	// Port B output for bar graph LED 8 bit out (0 for turn on for Kathod)
	DDRE = 0xFF;	// Port E output for bar graph LED section selection (1 for enable section - common Anode)
}

/*******************************************************************************
 Set char attribute for additional effect
*******************************************************************************/
void MultiSegmentDisplay::setAttribute(uint8_t bufferIndex, uint8_t attribute)
{
	MultiSegmentDisplay::segmentAttributeBuffer[bufferIndex] = attribute;
}

void MultiSegmentDisplay::setAttribute(uint8_t bufferIndexStart, uint8_t bufferIndexEnd, uint8_t attribute)
{
	for (uint8_t i = bufferIndexStart; i <= bufferIndexEnd; i++) {
		MultiSegmentDisplay::segmentAttributeBuffer[i] = attribute;
	}
}


/*******************************************************************************
 max 10bit = 1024 (Not linear)
 10 bit bar display needed and it should be right to left filling need.
*******************************************************************************/
void MultiSegmentDisplay::setAccelerator(uint16_t accelerator)
{
	//const int16_t ACCELERATOR_FULL = 700;
	//const int16_t ACCELERATOR_BEFORE_FULL = 500;
	
	//static const int16_t aceelToBarLevel[11] = {
		//~0b0000000000,
		//~0b1000000000,
		//~0b1100000000,
		//~0b1110000000,
		//~0b1111000000,
		//~0b1111100000,
		//~0b1111110000,
		//~0b1111111000,
		//~0b1111111100,
		//~0b1111111110,
		//~0b1111111111,
	//};

	static const int16_t aceelToBarLevel[11] = {	// LSB bit is left most position in real LED bar.
		~0b1111111111,	// Full accelerator
		~0b1111111110,
		~0b1111111100,
		~0b1111111000,
		~0b1111110000,
		~0b1111100000,
		~0b1111000000,
		~0b1110000000,
		~0b1100000000,
		~0b1000000000,
		~0b0000000000,	// No accelerator
	};

	// Try to first 1 bit checking
	accelerator *= 0.64;	// MAX 800 --> 512 conversion for display scale.
	const int16_t MSB = 0b1000000000;
	uint8_t index = 10;		// Default clear LED
	// Check first 1 bit from MSB to LSB and set index value from 1's position
	for (int i = 0; i < 10; i++) {
		if ((accelerator & MSB) == MSB) {
			index = i;
			break;
		}
		accelerator = accelerator << 1;
	}

	int16_t barGraphOut = aceelToBarLevel[index];
	barGraphSegmentOutBuffer[1] = (barGraphSegmentOutBuffer[1] & 0b11111100) | ((barGraphOut >> 8) & 0b11) ;
	barGraphSegmentOutBuffer[0] = barGraphOut & 0xFF;
	
	
	// TODO: MAX value slow fading effect - 최고값을 약 1초가 유지 후 아래 단계로 이동하는 effect를 구현 해 보자!
	// 여기서는 MAX value 기록만 하고 실제 animation 등은 interrupt 에서 해야 정확하게 자동할 것임
	// small index value is represent as Hi acceleration on LEDs
	
	if ( maxAcceleratorIndex <= (10 - index)) {
		maxAcceleratorIndex = (10 - index);
		maxAcceleratorIndexUpdateTickTime = (tick_timer + 2000) % 60000;	//tick_timer ? (tick_timer - 1) : (60000 -1);
		startMoveDown = false;	// It will turn to true for next maxAcceleratorIndexUpdateTickTime == tick_timer time
	}
	//maxAcceleratorIndex = 10;
}

/*******************************************************************************
 Set rpm values
*******************************************************************************/
void MultiSegmentDisplay::setRpmData(uint16_t down, uint16_t current, uint16_t up)
{
	const int16_t RPM_TO_ONE_LED = 200;	// 200 rpm for one LED bar
	const int16_t MAX_LED_COUNT = 30;
	const uint8_t bitFillPattern[8] = {
		0b11111111,
		0b11111110,
		0b11111100,
		0b11111000,
		0b11110000,
		0b11100000,
		0b11000000,
		0b10000000,
	};
	
	// [0..6000] 까지 mapping 하고 6000 이상(가솔린 등)에 대해서는 6000RPM 까지만 표시 필요
	// 1차: current value 값에 대해서 LED 30개에 mapping 처리 필요.
	// barGraphSegmentOutBuffer[1] 의 상위 2bit 부터 채워야 한다!
	// 우선 30bit로 채운 뒤 다시 개별 byte로 옮겨야만 할 것임.
	
	int16_t ledCount = (current + 100) / RPM_TO_ONE_LED;	// +100 for round up
	if (ledCount > MAX_LED_COUNT) {
		ledCount = MAX_LED_COUNT;
	}
	uint32_t ledBitBuffer = ~0;
	uint8_t* byteLedBitBuffer = (uint8_t*)&ledBitBuffer;
	
	// Fill the RPM bar graph with optimized (byte + partial pattern)
	int8_t fullFillCount = ledCount / 8;
	int8_t partialFillCount = ledCount % 8;
	for (int8_t i = 0; i < fullFillCount; i++) {
		byteLedBitBuffer[i] = 0;
	}
	byteLedBitBuffer[fullFillCount] = bitFillPattern[partialFillCount];
	
	// Shift down RPM marking (On)
	downShiftPointSegmentIndex = 0;	// Turn off down shift pointing effect
	downShiftPointOffsetIndex = 0;
	if (down) {
		int16_t shiftDownIndex = (down + 100) / RPM_TO_ONE_LED;
		if (shiftDownIndex > MAX_LED_COUNT) {
			shiftDownIndex = MAX_LED_COUNT;
		}
		// OK just 1 bit turn on for this case
		ledBitBuffer &= ~((uint32_t)1 << shiftDownIndex);
		// Get last bit position for display fast flashing attribute
		// + 1 segment + 2bit for last bit position
		downShiftPointSegmentIndex = (shiftDownIndex + 10) / 8;	// Trun on down shift pointing effect + segment selection
		downShiftPointOffsetIndex = (shiftDownIndex + 10) % 8;
	}
	// Shift up RPM marking (Off)
	if (up && ledCount > 14) {
		int16_t shiftUpIndex = (up + 100) / RPM_TO_ONE_LED;
		if (shiftUpIndex > MAX_LED_COUNT) {
			shiftUpIndex = MAX_LED_COUNT;
		}
		ledBitBuffer |= (uint32_t)1 << shiftUpIndex;
	}
	
	/* OK now the ledBitBuffer is ready to use, Remapping to output buffer (10bit shift up for RPM section)
	           11 1111112222 2222333333
	   0123456701 2345670123 4567012345
	   ||||||||                  ||||||
	   +----------+                   +----------+
	              ||||||||                       |
	           11 1111111122 2222333333 3344444444
	   0123456701 2345670123 4567012345 6701234567
	   
	   
	   상위 2bit 살려서 +1 위치 목적지로 옮기고
	   하위 6bit << 2 후 목적위치로 6bit만 masking 하여 옮기기
	*/
	for (int8_t i = 0; i < 4; i++) {
		uint8_t low6Bit = byteLedBitBuffer[i] & 0b111111; // Get low 6bit
		barGraphSegmentOutBuffer[1+i] = (barGraphSegmentOutBuffer[1+i] & 0b00000011) | (low6Bit << 2);	// Move to upper 6bit
		if (i < 3) {
			uint8_t upper2Bit = byteLedBitBuffer[i] & 0b11000000; // Get low 6bit
			barGraphSegmentOutBuffer[2+i] = (barGraphSegmentOutBuffer[2+i] & 0b11111100) | (upper2Bit >> 6);
		}
	}
}


/*******************************************************************************
 Update 1 digit 7 segment display.
 It need 15 times for update all digit.
*******************************************************************************/
SIGNAL( SIG_OUTPUT_COMPARE0 )
{
	//static uint16_t tick_timer = 0;

	//---------------------------------------- 7 segment section
	static uint8_t digitIndex = 0;
	uint16_t activeSegment = 1 << digitIndex;
	SEGMENT_OUT = 0xFF;	// Turn off all LED before change digit scanning (prevent crosstalk)
	DIGIT_0_TO_7 = activeSegment & 0x00FF;
	DIGIT_8_TO_15 = (activeSegment & 0xFF00) >> 8;

	uint8_t& attr = MultiSegmentDisplay::segmentAttributeBuffer[digitIndex];
	bool showOrNot = true;
	if (attr & MultiSegmentDisplay::ATTR_BLINK) {
		if ((attr & MultiSegmentDisplay::ATTR_BLINK_FAST) == MultiSegmentDisplay::ATTR_BLINK_FAST) {
			showOrNot = (MultiSegmentDisplay::tick_timer % 300 > 150);
		} else {
			showOrNot = (MultiSegmentDisplay::tick_timer % 2000 > 1000);
		}
	}
	if (showOrNot) {
		SEGMENT_OUT = MultiSegmentDisplay::segmentOutBuffer[digitIndex];
	}

	// Shift to next LED segment	
	if (++digitIndex >= MultiSegmentDisplay::MAX_SEGMENT_COUNT) {
		digitIndex = 0;
	}

	//---------------------------------------- Bar-graph section
	static uint8_t barIndex = 0;
	uint8_t activeBartSegment = 1 << barIndex;
	BAR_SEGMENT_OUT = 0xFF;	// Turn off before shift the segment
	BAR_GRAPH_0_TO_7 = activeBartSegment;
	
	BAR_SEGMENT_OUT = MultiSegmentDisplay::barGraphSegmentOutBuffer[barIndex];
	// Additional effect for specific bit
	if (MultiSegmentDisplay::downShiftPointSegmentIndex) {
		if (barIndex == MultiSegmentDisplay::downShiftPointSegmentIndex) {
			bool bitShowOrNot = MultiSegmentDisplay::downShiftPointDimming ?
							(MultiSegmentDisplay::tick_timer % 50 > 40) : (MultiSegmentDisplay::tick_timer % 500 > 250);
			if (!bitShowOrNot) {
				BAR_SEGMENT_OUT |= (uint8_t)1 << MultiSegmentDisplay::downShiftPointOffsetIndex;
			}
		}
	}
	
	
	if ((!MultiSegmentDisplay::startMoveDown) &&
		(MultiSegmentDisplay::tick_timer == MultiSegmentDisplay::maxAcceleratorIndexUpdateTickTime)) {
		MultiSegmentDisplay::startMoveDown = true;	
	}
	if (MultiSegmentDisplay::startMoveDown) {
		static bool tickTock = false;
		if (tickTock != (MultiSegmentDisplay::tick_timer % 1000 > 500)) {
			tickTock = !tickTock;
			if (MultiSegmentDisplay::maxAcceleratorIndex > 0) {
				--MultiSegmentDisplay::maxAcceleratorIndex;
			}
		}
	}
	
	if (MultiSegmentDisplay::maxAcceleratorIndex) {
		static const int16_t aceelToBarLevel[11] = {	// LSB bit is left most position in real LED bar.
			~0b0000000001,	// Full accelerator
			~0b0000000010,
			~0b0000000100,
			~0b0000001000,
			~0b0000010000,
			~0b0000100000,
			~0b0001000000,
			~0b0010000000,
			~0b0100000000,
			~0b1000000000,
			~0b0000000000,	// No accelerator
		};
		
		if (barIndex == 0) {
			BAR_SEGMENT_OUT &= aceelToBarLevel[10-MultiSegmentDisplay::maxAcceleratorIndex] & 0xff;
		} else if (barIndex == 1) {
			BAR_SEGMENT_OUT &= (uint8_t)(aceelToBarLevel[10-MultiSegmentDisplay::maxAcceleratorIndex] >> 8);
		}
	}
	
	if (++barIndex >= MultiSegmentDisplay::MAX_BAR_SEGMENT_COUNT) {
		barIndex = 0;
	}
	
	//---------------------------------------- Common section
	if (++MultiSegmentDisplay::tick_timer >= 60000) {
		MultiSegmentDisplay::tick_timer = 0;
	}
}


/*******************************************************************************
 Long term timer signal
 It use for attribute auto aging function for clear the blink effect.
*******************************************************************************/
uint8_t MultiSegmentDisplay::blinkDelayCount = 0;
SIGNAL( SIG_OUTPUT_COMPARE2 )
{
	const uint8_t MAGIC_VALUE_ADJUST_1000MS = 60;
	
	if (++MultiSegmentDisplay::blinkDelayCount < MAGIC_VALUE_ADJUST_1000MS) {
		return;
	} else {
		MultiSegmentDisplay::blinkDelayCount = 0;
	}
	
	// May not need reset the timer count for interrupt
	
	// Invoking 10Hz timing for check attribute of remain times for blinking
	for (int i = 0; i < MultiSegmentDisplay::MAX_SEGMENT_COUNT; i++) {
		uint8_t& attr = MultiSegmentDisplay::segmentAttributeBuffer[i];
		uint8_t timeCount = attr & 0b00111111;
		if (timeCount == 0) {	// No remain time then rest the attribute to 0 (No blinking)
			attr = 0;
		} else if (timeCount != MultiSegmentDisplay::ATTR_BLINK_FOREVER) {				// Decrease time value
			attr = (attr & 0b11000000) | (timeCount - 1);
		}
	}
	
}

	//						 012345678901234
	//const char* testString = "0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_'abcdefghijklmnopqrstuvwxyz{|}";
	//segmentDisplay.setString(0, testString);
	//segmentDisplay.setSegmentData(0, segmentDisplay.RAW_H);
	//int8_t len = strlen(testString);
	//int8_t index = 0;
	//uint16_t copy_tick_timer;
	//uint32_t loopCount = 0;
	//while (true) {
	//if (loopCount++ % 0x7FFF == 0x7FFF - 1) {
	//segmentDisplay.setString(0, testString + index);
	//
	//char buffer[4];
	//sprintf(buffer, "%3d", index);
	//segmentDisplay.setString(12, buffer);
	//
	//if (++index > len - 15) {
	//index = 0;
	//}
	//
	//}
	//}

	//segmentDisplay.set3DigitValueWithDotPos(0, 322, 1);
	//segmentDisplay.set3DigitValueWithDotPos(3, 324, 1);
	//segmentDisplay.set3DigitValueWithDotPos(6, 306, 1);
	//segmentDisplay.set3DigitValueWithDotPos(9, 308, 1);
