// By SeungWon Lee - minor modification applied
//
// 2016.05.16 - Translate comment to english from deutsches
// 2016.05.01 - Add interrupt handler handling logic and function for CAN RX
//
// Original source code: http://www.mikrocontroller.net/topic/98697 mod version of Michael

#ifndef CAN_H
#define CAN_H

#include "config.h"	//#define F_CPU 16000000UL and more

// For compile with CPP environment
#ifdef __cplusplus
extern "C" {
#endif

#include <inttypes.h>
#include <stdbool.h>


#define setbit(ADR,BIT)   (ADR |= (1<<BIT))
#define clearbit(ADR,BIT) (ADR &=~(1<<BIT))
#define getbit(ADR, BIT)  (ADR &  (1<<BIT))

#define DLC_MSK			((1<<DLC3)|(1<<DLC2)|(1<<DLC1)|(1<<DLC0))            //! Mask for Data Length Coding
#define DLC				(DLC0)
#define can_get_dlc()   ((CANCDMOB & DLC_MSK) >> DLC)

// CAN objects
#define NUMBER_OF_MOBS	15		// Set the number of message objects
#define NOMOB			0xff	// Definition of invalid object index

// Operating modes
typedef enum { 
	CAN_MODE_DISABLED, CAN_MODE_TRANSMIT_DATA, CAN_MODE_TRANSMIT_REMOTE, CAN_MODE_RECEIVE_DATA, CAN_MODE_AUTO_REPLY
} CanMode;

// Interrupt modes
typedef enum {
	CAN_INTERRUPT_NONE, CAN_INTERRUPT_TX, CAN_INTERRUPT_RX, CAN_INTERRUPT_TXRX
} CanInterruptMode;

// Baud rate
typedef enum {
	CAN_BAUDRATE_100K, CAN_BAUDRATE_125K, CAN_BAUDRATE_200K, CAN_BAUDRATE_250K, CAN_BAUDRATE_500K, CAN_BAUDRATE_1000K
} CanBaudRate;

// Main function
int can_init(CanBaudRate baudrate, CanInterruptMode intmode, bool extendedId);
int can_deinit(void);
int can_enable_mob(uint8_t mob, uint8_t mode, uint32_t id, uint32_t idm);
int can_disable_mob(uint8_t mob);
int can_send_data(uint8_t mob, uint8_t length, uint8_t * data);
int can_send_remote(uint8_t mob);
void can_set_autoreply(uint8_t mob, uint8_t length, uint8_t *buf);

// Other helpful functions
int can_set_baudrate(CanBaudRate baudrate);
int can_set_interrupt(CanInterruptMode mode);
void can_get_mob(uint8_t mob);
uint32_t can_get_id(void);
void can_set_id_mask(uint32_t idm);
void can_set_id(uint32_t id);
int can_set_mode(uint8_t mode);
uint8_t can_get_mode(void);
void can_set_data(uint8_t length, uint8_t * data);
int can_get_data(uint8_t *msg);
void can_set_mob_interrupt(uint8_t object);
void can_clear_mob_interrupt(uint8_t mob);
uint8_t can_get_mob_interrupt(void);

void set_interrupt_handler(void (*handler)(void));

#ifdef __cplusplus
}
#endif

#endif /* CAN_H */
